"""Constants for the Goal Zero Yeti integration."""
import logging
from typing import Final

ATTRIBUTION = "Data provided by Goal Zero"
ATTR_DEFAULT_ENABLED = "default_enabled"

DOMAIN: Final = "goalzero"
DEFAULT_NAME = "Yeti"
MANUFACTURER = "Goal Zero"

LOGGER = logging.getLogger(__name__)
