"""Constants for pushover."""

from typing import Final

DOMAIN: Final = "pushover"
DATA_HASS_CONFIG: Final = "pushover_hass_config"
DEFAULT_NAME: Final = "Pushover"

ATTR_ATTACHMENT: Final = "attachment"
ATTR_URL: Final = "url"
ATTR_URL_TITLE: Final = "url_title"
ATTR_PRIORITY: Final = "priority"
ATTR_RETRY: Final = "retry"
ATTR_SOUND: Final = "sound"
ATTR_HTML: Final = "html"
ATTR_CALLBACK_URL: Final = "callback_url"
ATTR_EXPIRE: Final = "expire"
ATTR_TIMESTAMP: Final = "timestamp"

CONF_USER_KEY: Final = "user_key"
