"""Eight Sleep constants."""
DOMAIN = "eight_sleep"

HEAT_ENTITY = "heat"
USER_ENTITY = "user"

NAME_MAP = {
    "current_sleep": "Sleep Session",
    "current_sleep_fitness": "Sleep Fitness",
    "last_sleep": "Previous Sleep Session",
}

SERVICE_HEAT_SET = "heat_set"

ATTR_TARGET = "target"
ATTR_DURATION = "duration"
