"""The Philips TV constants."""

DOMAIN = "philips_js"
CONF_SYSTEM = "system"
CONF_ALLOW_NOTIFY = "allow_notify"

CONST_APP_ID = "homeassistant.io"
CONST_APP_NAME = "Home Assistant"

TRIGGER_TYPE_TURN_ON = "turn_on"
