"""Constants for the Anthem A/V Receivers integration."""
ANTHEMAV_UDATE_SIGNAL = "anthemav_update"
CONF_MODEL = "model"
DEFAULT_NAME = "Anthem AV"
DEFAULT_PORT = 14999
DOMAIN = "anthemav"
MANUFACTURER = "Anthem"
DEVICE_TIMEOUT_SECONDS = 4.0
