"""Constants for the simplepush integration."""

from typing import Final

DOMAIN: Final = "simplepush"
DEFAULT_NAME: Final = "simplepush"
DATA_HASS_CONFIG: Final = "simplepush_hass_config"

ATTR_ENCRYPTED: Final = "encrypted"
ATTR_EVENT: Final = "event"

CONF_DEVICE_KEY: Final = "device_key"
CONF_SALT: Final = "salt"
