"""Constants for the pushbullet integration."""

from typing import Final

DOMAIN: Final = "pushbullet"
DEFAULT_NAME: Final = "Pushbullet"
DATA_HASS_CONFIG: Final = "pushbullet_hass_config"
DATA_UPDATED: Final = "pushbullet_data_updated"

ATTR_URL: Final = "url"
ATTR_FILE: Final = "file"
ATTR_FILE_URL: Final = "file_url"
